scriptName LZIOUtil Hidden

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; AUTHOR: Lord Zapharos                                                                         ;;
;; VERSION: 1.0                                                                                  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This SKSE plugin allows you to read and write line-based text files anywhere in the Skyrim    ;;
;; application directory. Unlike other storage solutions, the files created by this plugin are   ;;
;; NOT tied to your save file in any way. All function calls are buffered, meaning that you can  ;;
;; rapidly read and and write extremely large volumes of text without crashing the game.         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given file exists.
;; This file is relative to your main Skyrim directory (the folder containing TESV.exe).
;; You can use ".." to move up a directory e.g. "../myDir".
;; You should always use forward path separators e.g. "my/path" instead of "my\path".
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function ExistsFile(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loads a file for either reading or writing (but not both!).
;; When you are done using the file, call CloseFile() to save and remove it from memory.
;; If you forget to call CloseFile(), YOUR SCRIPT WILL BECOME BLOATED AND EVENTUALLY CRASH.
;; You have been warned!
;;
;; It is not possible to read from and write to a file simultaneously.
;; If you are reading a file and later decide to write to it, you must close the file first.
;; Using "isReadOnly == True" will create a read-only file; using "isReadOnly == False" will create
;;  a write-only file instead.
;; If the file you want to write does not yet exist, it will be immediately created.
;;
;; When you open a file for writing, "append == True" will cause NextLine() to append its contents
;;  to the very end of the file; in other words, the original contents will be retained.
;; If you choose "append == False", the file's current contents (if any) will be erased,
;;  and NextLine() will start writing from the beginning.
;;
;; To erase a file's contents, call this function with "append == True", then call CloseFile().
;;
;; The buffer size indicates how many lines are read at once from the file,
;;  or how many calls to WriteLine() must be made before lines are written.
;; Larger values tend to improve script performance, but may cause intermittent lag
;;  when a large amount of data is finally written.
;; A good "starter value" is 100.
;;
;; This function will return True if the file was loaded/created and can be used.
;; If this function returns False, an error has occurred and your script should act accordingly.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function LoadFile(string filename, bool isReadOnly, bool append, int bufferSize) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Closes the given file, saving any changes as needed.
;; This function must be called exactly once for each unique file loaded via LoadFile().
;; Failure to close files WILL bloat your script and cause the game to eventually crash.
;;
;; This function will return True if the file was closed.
;; If this function returns False, either the file has not yet been loaded
;;  or an error has occurred and your script should act accordingly.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function CloseFile(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deletes the given file.
;; If the file is already open, it will be closed first.
;;
;; This function will return True if the file was successfully deleted.
;; If this function returns False, either the file does not exist
;;  or an error has occurred and your script should act accordingly.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function DeleteAnyFile(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given file has been loaded.
;; You must separately call IsGood() to determine whether this file can actually be read/written.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsLoaded(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given file is ready for reading/writing.
;; If this function returns False, either the file has not yet been loaded
;;  or an error occurred on a prior function call and your script should act accordingly.
;; This function is typically only used for debugging e.g. if NextLine() repeatedly fails.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsGood(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given file is only able to be read.
;; If this function returns False, either the file is writable or it has not yet been loaded.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsReadOnly(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the given file is only able to be written.
;; If this function returns False, either the file is readable or it has not yet been loaded.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function IsWriteOnly(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Determines whether the (readable) file has at least one more line that can be read.
;; This function will always return False for writable-only files,
;;  or for files that have not yet been loaded.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function HasNextLine(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Obtains the next line from the (readable) file.
;; If there are no more available lines, or if the file is writable-only,
;;  this function will return an empty string "".
;; As such, it is useful to call HasNextLine() prior to this function.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string Function NextLine(string filename) global native


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Writes a line of text to the (writable) file.
;; A newline character will be automatically appended to the text you provide to this function.
;; As such, calling WriteLine("myFile.txt", "abc") followed by WriteLine("myFile.txt", "123")
;;  will produce the following file contents:
;;
;; myFile.txt
;;
;;  abc
;;  123
;;
;; This function will return True if the line was successfully written.
;; If this function returns False, either the given file is not writable, has not yet been loaded,
;;  or an error has occurred and your script should act accordingly.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
bool Function WriteLine(string filename, string text) global native
