#include "LZIOUtil.h"
#include <unordered_map>

namespace lznsIOUtil {
	/* The hashmap linking all filenames to their associated file instances. */
	static std::unordered_map<std::string, ExternalFile*> s_fileMap;

	/* Obtains the entry corresponding to a loaded file. */
	std::unordered_map<std::string, ExternalFile*>::const_iterator 
			get(BSFixedString filename) {
		std::string key(filename.data);
		return s_fileMap.find(key);
	}

	bool RegisterFuncs(VMClassRegistry* registry) {
		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("ExistsFile", "LZIOUtil", lznsIOUtil::ExistsFile, registry));

		registry->RegisterFunction(
			new NativeFunction4 <StaticFunctionTag, bool, BSFixedString, bool, bool, UInt32>("LoadFile", "LZIOUtil", lznsIOUtil::LoadFile, registry));

		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("CloseFile", "LZIOUtil", lznsIOUtil::CloseFile, registry));

		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("DeleteAnyFile", "LZIOUtil", lznsIOUtil::DeleteAnyFile, registry));
		
		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("IsLoaded", "LZIOUtil", lznsIOUtil::IsLoaded, registry));
		
		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("IsGood", "LZIOUtil", lznsIOUtil::IsGood, registry));
		
		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("IsReadOnly", "LZIOUtil", lznsIOUtil::IsReadOnly, registry));
		
		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("IsWriteOnly", "LZIOUtil", lznsIOUtil::IsWriteOnly, registry));

		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, bool, BSFixedString>("HasNextLine", "LZIOUtil", lznsIOUtil::HasNextLine, registry));

		registry->RegisterFunction(
			new NativeFunction1 <StaticFunctionTag, BSFixedString, BSFixedString>("NextLine", "LZIOUtil", lznsIOUtil::NextLine, registry));

		registry->RegisterFunction(
			new NativeFunction2 <StaticFunctionTag, bool, BSFixedString, BSFixedString>("WriteLine", "LZIOUtil", lznsIOUtil::WriteLine, registry));

		return true;
	}



	bool ExistsFile(StaticFunctionTag* base, BSFixedString filename) {
		std::string temp(filename.data);
		std::filesystem::path filepath(temp);
		return std::filesystem::exists(filepath);
	}

	bool LoadFile(StaticFunctionTag* base, BSFixedString filename, bool isReadOnly, 
			bool append, UInt32 bufferSize) {
		std::string key(filename.data);
		if (s_fileMap.find(key) != s_fileMap.end()) {
			return true; // file is already loaded
		}
		
		ExternalFile* f = new ExternalFile(
			std::string(filename.data), 
			isReadOnly, 
			append, 
			bufferSize);

		if (!f->isGood()) {
			delete f;
			return false;
		}
		
		s_fileMap[key] = f;
		return true;
	}

	bool CloseFile(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);

		bool isClosed = false;
		if (f != s_fileMap.end()) {
			isClosed = f->second->close();
			s_fileMap.erase(f);
		}
		return isClosed;
	}

	bool DeleteAnyFile(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		if (f == s_fileMap.end() && LoadFile(base, filename, true, false, 1)) {
			f = get(filename);
		}

		bool isDeleted = false;
		if (f != s_fileMap.end()) {
			isDeleted = (f->second->close() && f->second->deleteFile());
			s_fileMap.erase(f);
		}
		return isDeleted;
	}

	bool IsLoaded(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		return f != s_fileMap.end();
	}

	bool IsGood(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		return f != s_fileMap.end() && f->second->isGood();
	}

	bool IsReadOnly(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		return f != s_fileMap.end() && f->second->isReadOnly();
	}

	bool IsWriteOnly(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		return f != s_fileMap.end() && !f->second->isReadOnly();
	}

	bool HasNextLine(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		return f != s_fileMap.end() && f->second->hasNextLine();
	}

	BSFixedString NextLine(StaticFunctionTag* base, BSFixedString filename) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		
		std::string next;
		if (f != s_fileMap.end()) {
			next = f->second->getNextLine();
		}
		return next.c_str();
	}

	bool WriteLine(StaticFunctionTag* base, BSFixedString filename, BSFixedString text) {
		std::unordered_map<std::string, ExternalFile*>::const_iterator f = get(filename);
		if (f == s_fileMap.end()) {
			return false;
		}

		return f->second->writeLine(std::string(text.data));
	}



	ExternalFile::ExternalFile(std::string f, bool isReadOnly, bool append, int bufferSize) 
			: _isGood(true), 
			_isReadOnly(isReadOnly),
			_append(append), 
			_linePointer((isReadOnly) ? bufferSize : 0),
			_bufferSize(bufferSize),
			_filepath(f),
			_lineBuffer(NULL) {
		bool fileExists = std::filesystem::exists(_filepath);
		_filepath.make_preferred();

		_MESSAGE("Loading file '%s'...", _filepath.generic_string().c_str());
		if (!fileExists && _isReadOnly) {
			_MESSAGE("  File does not exist! Aborting load.");
			_isGood = false;
			return;
		} else if (bufferSize <= 0) {
			_MESSAGE("  Illegal buffer size '%d'!", bufferSize);
			_isGood = false;
			return;
		}

		_lineBuffer = new std::string[_bufferSize];
		_lineBuffer[0] = { EOL };

		s_dataLock.Enter();
		try {
			if (_isReadOnly) {
				_inputStream.open(_filepath, std::ofstream::in);
				if (!_inputStream.is_open()) {
					_isGood = false;
				}
			} else if (_append) {
				_outputStream.open(_filepath, std::ofstream::app | std::ofstream::out);
				if (!_outputStream.is_open()) {
					_isGood = false;
				}
			} else {
				_outputStream.open(_filepath, std::ofstream::trunc | std::ofstream::out);
				if (!_outputStream.is_open()) {
					_isGood = false;
				}
			}

			if (_isGood) {
				_MESSAGE("  File successfully loaded.");
			} else {
				_MESSAGE("  Could not open or create file.");
			}

		} catch (const std::exception& e) {
			_isGood = false;
			_MESSAGE("  Error reading or creating file:");
			_MESSAGE("  %s\n", e.what());
		}
		s_dataLock.Leave();
	}

	ExternalFile::~ExternalFile() {
		delete _lineBuffer;
	}

	bool ExternalFile::isGood() {
		return _isGood;
	}

	bool ExternalFile::isReadOnly() {
		return _isReadOnly;
	}

	bool ExternalFile::close() {
		try {
			if (_isReadOnly && _inputStream.is_open()) {
				_inputStream.close();
			} else if (!_isReadOnly && _outputStream.is_open()) {
				dumpBuffer();
				_outputStream.close();
			}

			_isGood = false;
			_MESSAGE("Closed file '%s'.", _filepath.generic_string().c_str());
			return true;

		} catch (const std::exception& e) {
			_isGood = false;
			_MESSAGE("  Error closing file:");
			_MESSAGE("  %s\n", e.what());
			return false;
		}
	}

	bool ExternalFile::deleteFile() {
		if (_isGood) {
			return false;
		}

		try {
			return std::filesystem::remove(_filepath);
		} catch (const std::exception& e) {
			_MESSAGE("  Error deleting file:");
			_MESSAGE("  %s\n", e.what());
			return false;
		}
	}

	bool ExternalFile::hasNextLine() {
		if (!_isGood || !_isReadOnly) {
			return false;
		}

		if (_linePointer == _bufferSize) {
			readBuffer();
		}

		std::string s = _lineBuffer[_linePointer];
		return s.length() == 0 || s[0] != EOL;
	}

	std::string ExternalFile::getNextLine() {
		if (!_isGood || !_isReadOnly) {
			return "";
		}

		if (_linePointer == _bufferSize) {
			readBuffer();
		}
		return _lineBuffer[_linePointer++];
	}

	bool ExternalFile::writeLine(std::string text) {
		if (!_isGood || _isReadOnly) {
			return false;
		}

		if (_linePointer == _bufferSize) {
			dumpBuffer();
		}

		_lineBuffer[_linePointer++] = text;
		if (_linePointer < _bufferSize) {
			_lineBuffer[_linePointer] = { EOL };
		}
		return true;
	}



	void ExternalFile::readBuffer() {
		if (!_isGood || !_isReadOnly) {
			return;
		}

		s_dataLock.Enter();
		try {
			_linePointer = 0;

			std::string temp;
			while (_linePointer < _bufferSize && std::getline(_inputStream, temp)) {
				_lineBuffer[_linePointer++] = temp;
			}

			if (_linePointer < _bufferSize) {
				_lineBuffer[_linePointer] = { EOL };
			}
			_linePointer = 0;

		} catch (const std::exception& e) {
			_isGood = false;
			_MESSAGE("  Error reading file:");
			_MESSAGE("  %s\n", e.what());
		}
		s_dataLock.Leave();
	}

	void ExternalFile::dumpBuffer() {
		if (!_isGood || _isReadOnly) {
			return;
		}

		s_dataLock.Enter();
		try {
			_linePointer = 0;

			std::string temp;
			while (_linePointer < _bufferSize) {
				temp = _lineBuffer[_linePointer++];
				if (temp[0] == EOL) {
					break;
				}

				_outputStream << temp << "\n";
			}
			_linePointer = 0;

		} catch (const std::exception& e) {
			_isGood = false;
			_MESSAGE("  Error writing file:");
			_MESSAGE("  %s\n", e.what());
		}
		s_dataLock.Leave();
	}
} 
