#include "skse/PapyrusNativeFunctions.h"
#include "common/ICriticalSection.h"
#include <filesystem>
#include <fstream>

namespace lznsIOUtil
{
	/* Registers all Papyrus functions into the plugin. */
	bool RegisterFuncs(VMClassRegistry* registry);



	/* PAPYRUS: Determines whether the given file exists (relative to the game directory). */
	bool ExistsFile(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Attempts to load the given file, returning false if something went wrong. */
	bool LoadFile(StaticFunctionTag* base, BSFixedString filename, bool isReadOnly, bool append, UInt32 bufferSize);

	/* PAPYRUS: Attempts to close the given file, returning false if something went wrong. */
	bool CloseFile(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Attempts to delete (and if necessary close) the given file. */
	bool DeleteAnyFile(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Determines whether the given file has been loaded by this plugin. */
	bool IsLoaded(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Determines whether the given file is ready for reading/writing. */
	bool IsGood(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Determines whether the given file is read-only. */
	bool IsReadOnly(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Determines whether the given file is write-only. */
	bool IsWriteOnly(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Determines whether the given file has another line that can be read. */
	bool HasNextLine(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Reads the next line from the given file (buffered). */
	BSFixedString NextLine(StaticFunctionTag* base, BSFixedString filename);

	/* PAPYRUS: Writes a buffered line of text to the given file, appended by a newline. */
	bool WriteLine(StaticFunctionTag* base, BSFixedString filename, BSFixedString text);



	class ExternalFile {
	private:
		static const char EOL = 7;

		bool _isGood;

		bool _isReadOnly;
		bool _append;
		int _linePointer;
		int _bufferSize;

		std::filesystem::path _filepath;
		std::ifstream _inputStream;
		std::ofstream _outputStream;
		std::string* _lineBuffer;

		ICriticalSection s_dataLock;

		/* Reads a file's lines into the string buffer, then resets the pointer. */
		void readBuffer();

		/* Dumps all pending lines to the file, then resets the pointer. */
		void dumpBuffer();

	public:
		/* Creates a new external file for either reading or writing (not both). */
		ExternalFile(std::string f, bool isReadOnly, bool append, int bufferSize);

		/* Destroys this external file. */
		~ExternalFile();

		/* Determines whether this file is ready for reading or writing. */
		bool isGood();

		/* Determines whether this file is read-only. */
		bool isReadOnly();

		/* Closes this file. Should be called prior to destruction. */
		bool close();

		/* Deletes this (closed) file. */
		bool deleteFile();

		/* Determines whether this file has another readable line. */
		bool hasNextLine();

		/* Obtains the next line from this file. */
		std::string getNextLine();

		/* Writes a line to this file (a newline will be automatically appended). */
		bool writeLine(std::string text);
	};
}
